import fs from 'fs-extra';
import path from 'node:path';
import { Log } from '@tsbb/typescript';
import babelPluginJsx from '@vue/babel-plugin-jsx';
import { transform } from './transform.js';
import { getOutputPath } from './utils.js';
import { getCjsTransformOption, getESMTransformOption } from './config.js';
export * from './utils.js';
export default async function compile(fileName, options = {}) {
    const { cjs = 'lib', esm = 'esm', envName, useVue = false } = options;
    const dt = getOutputPath(fileName, options);
    const log = new Log();
    log.name();
    if (options.sourceMaps === 'true') {
        options.sourceMaps = true;
    }
    let babelOptions = typeof esm === 'string' ? getESMTransformOption() : typeof cjs === 'string' ? getCjsTransformOption() : {};
    if (useVue) {
        // @ts-ignore
        babelOptions.plugins?.push(babelPluginJsx.default);
        babelOptions.sourceMaps = options.sourceMaps || babelOptions.sourceMaps;
    }
    babelOptions.babelrc = true;
    babelOptions.cwd = dt.projectDirectory;
    if (envName) {
        babelOptions = {};
        babelOptions.envName = envName;
    }
    if (typeof esm === 'string') {
        babelOptions.sourceFileName = path.relative(path.dirname(dt.esm.path), fileName);
        transform(fileName, { ...babelOptions })
            .then((result) => {
            fs.ensureFileSync(dt.esm.path);
            fs.writeFile(dt.esm.path, result?.code || '');
            log.icon('🐶').success(`┈┈▶ \x1b[32;1m${dt.folderFilePath}\x1b[0m => \x1b[34;1m${dt.esm.fileName}\x1b[0m`);
            if (babelOptions.sourceMaps === 'both' || babelOptions.sourceMaps) {
                if (result?.map) {
                    const sourceMapPath = path.join(dt.esm.path + '.map');
                    fs.writeFileSync(sourceMapPath, JSON.stringify(result?.map, null, 2));
                    log
                        .icon('🐶')
                        .success(`┈┈▶ \x1b[32;1m${dt.folderFilePath}\x1b[0m => \x1b[34;1m${path.relative(dt.projectDirectory, sourceMapPath)}\x1b[0m`);
                }
            }
        })
            .catch((error) => {
            if (error instanceof Error) {
                log.icon('\n🚨').error(`\x1b[33;1m ${error.message}\x1b[0m\n`);
            }
            else {
                log.icon('\n🚨').error(`\x1b[33;1m ${JSON.stringify(error)}\x1b[0m\n`);
            }
        });
    }
    if (typeof cjs === 'string') {
        babelOptions.sourceFileName = path.relative(path.dirname(dt.cjs.path), fileName);
        transform(fileName, { ...babelOptions })
            .then((result) => {
            fs.ensureFileSync(dt.cjs.path);
            fs.writeFile(dt.cjs.path, result?.code || '');
            log.icon('🐶').success(`┈┈▶ \x1b[33;1m${dt.folderFilePath}\x1b[0m => \x1b[33;1m${dt.cjs.fileName}\x1b[0m`);
            if (babelOptions.sourceMaps === 'both' || babelOptions.sourceMaps) {
                if (result?.map) {
                    const sourceMapPath = path.join(dt.cjs.path + '.map');
                    fs.writeFileSync(sourceMapPath, JSON.stringify(result?.map, null, 2));
                    log
                        .icon('🐶')
                        .success(`┈┈▶ \x1b[33;1m${dt.folderFilePath}\x1b[0m => \x1b[33;1m${path.relative(dt.projectDirectory, sourceMapPath)}\x1b[0m`);
                }
            }
        })
            .catch((error) => {
            if (error instanceof Error) {
                log.icon('\n🚨').error(`\x1b[33;1m ${error.message}\x1b[0m\n`);
            }
            else {
                log.icon('\n🚨').error(`\x1b[33;1m ${JSON.stringify(error)}\x1b[0m\n`);
            }
        });
    }
}
